==> syd-3.21.0.tar.gz: syd-v3.21.0/src/syd-aes.rs <==
//
// syd: seccomp and landlock based application sandbox with support for namespaces
// src/syd-aes.rs: AES-CTR Encryption and Decryption Utility
//
// Copyright (c) 2024 Ali Polatel <alip@chesswob.org>
//
// SPDX-License-Identifier: GPL-3.0-or-later

use std::{
    env,
    os::fd::{AsRawFd, FromRawFd, OwnedFd},
    process::ExitCode,
};

use anyhow::{Context, Result};
use getargs::{Opt, Options};
use nix::{
    errno::Errno,
    fcntl::{splice, OFlag, SpliceFFlags},
    sys::{
        select::{select, FdSet},
        socket::{sendmsg, AlgAddr, ControlMessage, MsgFlags},
    },
    unistd::pipe2,
};
use syd::hash::{
    aes_ctr_dec, aes_ctr_enc, aes_ctr_flush, aes_ctr_init, aes_ctr_setup, Key, IV, IV_SIZE,
    KEY_SIZE,
};

fn process_data(encrypting: bool, key: Key, iv: IV) -> Result<()> {
    let fd = aes_ctr_setup(&key).context("Error setting up Kernel Cryptography API connection.")?;
    let fd = aes_ctr_init(&fd).context("Error initializing AES-CTR {en,de}cryption socket.")?;

    if encrypting {
        aes_ctr_enc(&fd, &[], &iv, None).context("Error setting up AES-CTR encryption.")?;
    } else {
        aes_ctr_dec(&fd, &[], &iv, true).context("Error setting up AES-CTR decryption.")?;
    }

    let (pipe_rd, pipe_wr) = pipe2(OFlag::O_CLOEXEC)
        .map(|(pipe_rd, pipe_wr)| {
            // SAFETY: pipe2 returns valid FDs.
            (unsafe { OwnedFd::from_raw_fd(pipe_rd) }, unsafe {
                OwnedFd::from_raw_fd(pipe_wr)
            })
        })
        .context("Error creating splice pipe pair.")?;

    let mut nread = 0;
    'outer: loop {
        // Use splice to move data from standard input to pipe.
        match splice(
            nix::libc::STDIN_FILENO,
            None,
            pipe_wr.as_raw_fd(),
            None,
            nix::libc::PIPE_BUF,
            SpliceFFlags::empty(),
        ) {
            Ok(0) | Err(_) => break,
            Ok(n) => {
                loop {
                    let mut rfds = FdSet::new();
                    let mut wfds = FdSet::new();
                    let mut efds = FdSet::new();
                    rfds.insert(pipe_rd.as_raw_fd());
                    wfds.insert(fd.as_raw_fd());
                    efds.insert(pipe_rd.as_raw_fd());
                    efds.insert(fd.as_raw_fd());
                    match select(None, &mut rfds, &mut wfds, &mut efds, None) {
                        Ok(2) => {
                            // Use splice to move data from pipe to sock (AF_ALG socket)
                            if !matches!(
                                splice(
                                    pipe_rd.as_raw_fd(),
                                    None,
                                    fd.as_raw_fd(),
                                    None,
                                    n,
                                    SpliceFFlags::SPLICE_F_MORE,
                                ),
                                Ok(0) | Err(_)
                            ) {
                                nread += n;
                            }
                            break;
                        }
                        Ok(_) if efds.highest().is_some() => break 'outer,
                        Ok(_) if wfds.highest().is_none() => {
                            // Kernel decryption buffer is full.
                            // We must pause to write to standard output.
                            // SAFETY: Zero-out memory if decrypting.
                            aes_ctr_flush(&fd, &nix::libc::STDOUT_FILENO, nread, !encrypting)?;
                            nread = 0;
                            continue;
                        }
                        Ok(_) | Err(Errno::EINTR) => continue, // No file descriptors are ready.
                        Err(errno) => return Err(errno.into()),
                    }
                }
            }
        }

        // Signal the AF_ALG socket the end of the process (No MSG_MORE in flags).
        let cmsgs = [ControlMessage::AlgSetOp(if encrypting {
            &nix::libc::ALG_OP_ENCRYPT
        } else {
            &nix::libc::ALG_OP_DECRYPT
        })];
        sendmsg::<AlgAddr>(fd.as_raw_fd(), &[], &cmsgs, MsgFlags::empty(), None)?;

        if nread > 0 {
            // {En,De}crypt the final batch.
            // SAFETY: Zero-out memory if decrypting.
            aes_ctr_flush(&fd, &nix::libc::STDOUT_FILENO, nread, !encrypting)?;
        }
    }

    Ok(())
}

#[allow(clippy::disallowed_methods)]
fn main() -> Result<ExitCode> {
    #[allow(clippy::disallowed_methods)]
    syd::set_sigpipe_dfl().expect("SIGPIPE");

    let args = env::args().skip(1).collect::<Vec<_>>();
    let mut opts = Options::new(args.iter().map(String::as_str));
    let mut encrypt: Option<bool> = None;
    let mut key_hex: Option<&str> = None;
    let mut iv_hex: Option<&str> = None;

    while let Some(opt) = opts.next_opt().expect("next opt") {
        match opt {
            Opt::Short('h') => {
                help();
                return Ok(ExitCode::SUCCESS);
            }
            Opt::Short('e') => {
                encrypt = Some(true);
            }
            Opt::Short('d') => {
                encrypt = Some(false);
            }
            Opt::Short('k') => {
                key_hex = opts.value().ok();
            }
            Opt::Short('i') => {
                iv_hex = opts.value().ok();
            }
            _ => {
                eprintln!("Unknown option: {opt:?}");
                help();
                return Ok(ExitCode::FAILURE);
            }
        }
    }

    if encrypt.is_none() || key_hex.is_none() || iv_hex.is_none() {
        eprintln!("Error: -e or -d and both -k and -i options are required.");
        help();
        return Ok(ExitCode::FAILURE);
    }

    let key = match const_hex::decode(key_hex.unwrap()) {
        Ok(key) => {
            if key.len() != KEY_SIZE {
                eprintln!("Error: Key must be 512 bits (64 bytes) in length.");
                return Ok(ExitCode::FAILURE);
            }
            Key::new(key.try_into().unwrap())
        }
        Err(err) => {
            eprintln!("Error: Key must be valid hex: {err}");
            return Ok(ExitCode::FAILURE);
        }
    };

    let iv = match const_hex::decode(iv_hex.unwrap()) {
        Ok(iv) => {
            if iv.len() != IV_SIZE {
                eprintln!("Error: IV must be 128 bits (16 bytes) in length.");
                return Ok(ExitCode::FAILURE);
            }
            iv
        }
        Err(err) => {
            eprintln!("Error: IV must be valid hex: {err}");
            return Ok(ExitCode::FAILURE);
        }
    };
    let iv = IV::new(iv.try_into().unwrap());

    process_data(encrypt.unwrap(), key, iv)
        .map(|_| ExitCode::SUCCESS)
        .context("Error processing data!")
}

fn help() {
    println!("Usage: syd-aes [-h] -e|-d -k <hex-encoded key> -i <hex-encoded iv>");
    println!("AES-CTR Encryption and Decryption Utility");
    println!("Reads from standard input and writes to standard output.");
    println!("  -h        Print this help message and exit");
    println!("  -e        Encrypt the input data");
    println!("  -d        Decrypt the input data");
    println!("  -k <key>  Hex-encoded key (256 bits for AES-CTR)");
    println!("  -i <iv>   Hex-encoded IV (128 bits for AES-CTR)");
}

==> syd-3.22.0.tar.gz: syd-v3.22.0/src/syd-aes.rs <==
//
// syd: seccomp and landlock based application sandbox with support for namespaces
// src/syd-aes.rs: AES-CTR Encryption and Decryption Utility
//
// Copyright (c) 2024 Ali Polatel <alip@chesswob.org>
//
// SPDX-License-Identifier: GPL-3.0-or-later

use std::{
    env,
    os::fd::{AsRawFd, FromRawFd, OwnedFd},
    process::ExitCode,
};

use anyhow::{Context, Result};
use getargs::{Opt, Options};
use nix::{
    errno::Errno,
    fcntl::{splice, OFlag, SpliceFFlags},
    sys::{
        select::{select, FdSet},
        socket::{sendmsg, AlgAddr, ControlMessage, MsgFlags},
    },
    unistd::pipe2,
};
use syd::hash::{
    aes_ctr_dec, aes_ctr_enc, aes_ctr_flush, aes_ctr_init, aes_ctr_setup, Key, IV, IV_SIZE,
    KEY_SIZE,
};

fn process_data(encrypting: bool, key: Key, iv: IV) -> Result<()> {
    let fd = aes_ctr_setup(&key)
        .map_err(anyhow::Error::msg)
        .context("Error setting up Kernel Cryptography API connection.")?;
    let fd = aes_ctr_init(&fd)
        .map_err(anyhow::Error::msg)
        .context("Error initializing AES-CTR {en,de}cryption socket.")?;

    if encrypting {
        aes_ctr_enc(&fd, &[], &iv, None)
            .map_err(anyhow::Error::msg)
            .context("Error setting up AES-CTR encryption.")?;
    } else {
        aes_ctr_dec(&fd, &[], &iv, true)
            .map_err(anyhow::Error::msg)
            .context("Error setting up AES-CTR decryption.")?;
    }

    let (pipe_rd, pipe_wr) = pipe2(OFlag::O_CLOEXEC)
        .map(|(pipe_rd, pipe_wr)| {
            // SAFETY: pipe2 returns valid FDs.
            (unsafe { OwnedFd::from_raw_fd(pipe_rd) }, unsafe {
                OwnedFd::from_raw_fd(pipe_wr)
            })
        })
        .map_err(anyhow::Error::msg)
        .context("Error creating splice pipe pair.")?;

    let mut nread = 0;
    'outer: loop {
        // Use splice to move data from standard input to pipe.
        match splice(
            nix::libc::STDIN_FILENO,
            None,
            pipe_wr.as_raw_fd(),
            None,
            nix::libc::PIPE_BUF,
            SpliceFFlags::empty(),
        ) {
            Ok(0) | Err(_) => break,
            Ok(n) => {
                loop {
                    let mut rfds = FdSet::new();
                    let mut wfds = FdSet::new();
                    let mut efds = FdSet::new();
                    rfds.insert(pipe_rd.as_raw_fd());
                    wfds.insert(fd.as_raw_fd());
                    efds.insert(pipe_rd.as_raw_fd());
                    efds.insert(fd.as_raw_fd());
                    match select(None, &mut rfds, &mut wfds, &mut efds, None) {
                        Ok(2) => {
                            // Use splice to move data from pipe to sock (AF_ALG socket)
                            if !matches!(
                                splice(
                                    pipe_rd.as_raw_fd(),
                                    None,
                                    fd.as_raw_fd(),
                                    None,
                                    n,
                                    SpliceFFlags::SPLICE_F_MORE,
                                ),
                                Ok(0) | Err(_)
                            ) {
                                nread += n;
                            }
                            break;
                        }
                        Ok(_) if efds.highest().is_some() => break 'outer,
                        Ok(_) if wfds.highest().is_none() => {
                            // Kernel decryption buffer is full.
                            // We must pause to write to standard output.
                            // SAFETY: Zero-out memory if decrypting.
                            aes_ctr_flush(&fd, &nix::libc::STDOUT_FILENO, nread, !encrypting)
                                .map_err(anyhow::Error::msg)?;
                            nread = 0;
                            continue;
                        }
                        Ok(_) | Err(Errno::EINTR) => continue, // No file descriptors are ready.
                        Err(errno) => return Err(anyhow::Error::msg(errno)),
                    }
                }
            }
        }

        // Signal the AF_ALG socket the end of the process (No MSG_MORE in flags).
        let cmsgs = [ControlMessage::AlgSetOp(if encrypting {
            &nix::libc::ALG_OP_ENCRYPT
        } else {
            &nix::libc::ALG_OP_DECRYPT
        })];
        sendmsg::<AlgAddr>(fd.as_raw_fd(), &[], &cmsgs, MsgFlags::empty(), None)
            .map_err(anyhow::Error::msg)?;

        if nread > 0 {
            // {En,De}crypt the final batch.
            // SAFETY: Zero-out memory if decrypting.
            aes_ctr_flush(&fd, &nix::libc::STDOUT_FILENO, nread, !encrypting)
                .map_err(anyhow::Error::msg)?;
        }
    }

    Ok(())
}

#[allow(clippy::disallowed_methods)]
fn main() -> Result<ExitCode> {
    #[allow(clippy::disallowed_methods)]
    syd::set_sigpipe_dfl().expect("SIGPIPE");

    let args = env::args().skip(1).collect::<Vec<_>>();
    let mut opts = Options::new(args.iter().map(String::as_str));
    let mut encrypt: Option<bool> = None;
    let mut key_hex: Option<&str> = None;
    let mut iv_hex: Option<&str> = None;

    while let Some(opt) = opts.next_opt().expect("next opt") {
        match opt {
            Opt::Short('h') => {
                help();
                return Ok(ExitCode::SUCCESS);
            }
            Opt::Short('e') => {
                encrypt = Some(true);
            }
            Opt::Short('d') => {
                encrypt = Some(false);
            }
            Opt::Short('k') => {
                key_hex = opts.value().ok();
            }
            Opt::Short('i') => {
                iv_hex = opts.value().ok();
            }
            _ => {
                eprintln!("Unknown option: {opt:?}");
                help();
                return Ok(ExitCode::FAILURE);
            }
        }
    }

    if encrypt.is_none() || key_hex.is_none() || iv_hex.is_none() {
        eprintln!("Error: -e or -d and both -k and -i options are required.");
        help();
        return Ok(ExitCode::FAILURE);
    }

    let key = match const_hex::decode(key_hex.unwrap()) {
        Ok(key) => {
            if key.len() != KEY_SIZE {
                eprintln!("Error: Key must be 512 bits (64 bytes) in length.");
                return Ok(ExitCode::FAILURE);
            }
            Key::new(key.try_into().unwrap())
        }
        Err(err) => {
            eprintln!("Error: Key must be valid hex: {err}");
            return Ok(ExitCode::FAILURE);
        }
    };

    let iv = match const_hex::decode(iv_hex.unwrap()) {
        Ok(iv) => {
            if iv.len() != IV_SIZE {
                eprintln!("Error: IV must be 128 bits (16 bytes) in length.");
                return Ok(ExitCode::FAILURE);
            }
            iv
        }
        Err(err) => {
            eprintln!("Error: IV must be valid hex: {err}");
            return Ok(ExitCode::FAILURE);
        }
    };
    let iv = IV::new(iv.try_into().unwrap());

    process_data(encrypt.unwrap(), key, iv)
        .map(|_| ExitCode::SUCCESS)
        .context("Error processing data!")
}

fn help() {
    println!("Usage: syd-aes [-h] -e|-d -k <hex-encoded key> -i <hex-encoded iv>");
    println!("AES-CTR Encryption and Decryption Utility");
    println!("Reads from standard input and writes to standard output.");
    println!("  -h        Print this help message and exit");
    println!("  -e        Encrypt the input data");
    println!("  -d        Decrypt the input data");
    println!("  -k <key>  Hex-encoded key (256 bits for AES-CTR)");
    println!("  -i <iv>   Hex-encoded IV (128 bits for AES-CTR)");
}
